﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using Infragistics.Olap;
using Infragistics.Olap.Data;
using Infragistics.Olap.Data.Base;
using Infragistics.Olap.Xmla;
using System.Linq;
using Query = Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Query;


namespace OlapAxisWithXMLA
{
    public class AdomdCommand : XmlaCommand
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="connection"></param>
        public AdomdCommand(XmlaConnection connection)
            : base(connection)
        {
            IncludeAllMemberInResults = true;
        }
        public bool IncludeAllMemberInResults { get; set; }
        public bool IsUpdateDisabled { get; set; }

        /// <summary>
        /// 
        /// </summary>
        public ObservableCollection<OlapFilter> ForeignFilters { get; set; }

        /// <summary>
        /// Executes a command and passes the result to the calling thread
        /// </summary>
        /// <param name="query"></param>
        /// <param name="connection"></param>
        public override void ExecuteCommandAsync(IMdxQuery query, XmlaConnection connection)
        {
            var adomdCommandExecutions = new List<AdomdCommandExecution>
                                             {
                                                 new AdomdCommandExecution
                                                     {
                                                         Query = query,
                                                         Connection = connection,
                                                         RemoveFilters = false
                                                     }
                                             };
            foreach (var command in adomdCommandExecutions)
            {
                var commandGroup = new AdomdCommandGroup
                {
                    AdomdCommandExecutions = adomdCommandExecutions,
                    CommandIndex = adomdCommandExecutions.IndexOf(command)
                };
                ExecuteAdomdCommand(commandGroup);
            }
        }

        /// <summary>
        /// executes a particular command in a group
        /// </summary>
        /// <param name="adomdCommandGroupObject"></param>
        private void ExecuteAdomdCommand(object adomdCommandGroupObject)
        {
            var adomdCommandGroup = (AdomdCommandGroup)adomdCommandGroupObject;
            var adomdCommandExecution = adomdCommandGroup.AdomdCommandExecutions[adomdCommandGroup.CommandIndex];
            adomdCommandExecution.Result = GetCommandResult(adomdCommandExecution.Query, adomdCommandExecution.Connection,
                                          adomdCommandExecution.RemoveFilters);
            //return adomdCommandGroup.AdomdCommandExecutions;
            AdomdCommandExecutionComplete(adomdCommandGroup.AdomdCommandExecutions);
        }

        void AdomdCommandExecutionComplete(IList<AdomdCommandExecution> commandExecutions)
        {
            if (commandExecutions != null)
            {
                // call the method for dealing with multiple results
                OnExecuteCommandCompleted(commandExecutions.Select(r => new ExecuteCommandCompletedEventArgs(r.Result, null, CancelPending,
                                                                               r.Query)).ToList());
                // call the method for dealing with a single result
                OnExecuteCommandCompleted(new ExecuteCommandCompletedEventArgs(commandExecutions[0].Result, null, CancelPending,
                                                                               commandExecutions[0].Query));
            }
        }

        /// <summary>
        /// Executed when all result sets are retrieved for query
        /// </summary>
        public event System.EventHandler<ExecuteMultipleCommandsCompletedEventArgs> ExecuteMultipleCommandsCompleted;

        /// <summary>
        /// Raises event for multiple results from cube for a single query
        /// </summary>
        /// <param name="executeCommandCompletedEventArgs"></param>
        protected virtual void OnExecuteCommandCompleted(List<ExecuteCommandCompletedEventArgs> executeCommandCompletedEventArgs)
        {
            if (ExecuteMultipleCommandsCompleted != null)
            {
                ExecuteMultipleCommandsCompleted(this,
                                                 new ExecuteMultipleCommandsCompletedEventArgs
                                                 {
                                                     ExecuteCommandCompletedEventArgs =
                                                         executeCommandCompletedEventArgs
                                                 });
            }
        }

        private IResult GetCommandResult(IMdxQuery query, XmlaConnection connection, bool removeFilters = false)
        {
            IResult result = null;

            if (IsUpdateDisabled)
            {
                result = GenerateResult(null, null, null, query.Cube);
            }
            else
            {
                var newQuery = (Query)query;

                var commandText = "";

                // create one of two queries: one which is the query reflected by choices in the data selector, 
                // and one which is an "unfiltered" version of the user selected query
                if (removeFilters)
                {
                    var unfilteredQuery = new Query(CopyAxis(newQuery.RowAxis, ForeignFilters, false),
                                                    CopyAxis(newQuery.ColumnAxis, ForeignFilters, false),
                                                    CopyAxis(newQuery.SlicerAxis, ForeignFilters, false),
                                                    newQuery.MdxSettings,
                                                    newQuery.
                                                        CalculatedMembersCache,
                                                    newQuery.Cube);
                    commandText = GenerateCommand(unfilteredQuery);
                }
                else
                {
                    commandText = GenerateCommand(newQuery);
                }

                // remove "All" if necessary
                if (!IncludeAllMemberInResults)
                {
                    var noAllQuery = new Query(CopyAxisWithoutAllMember(newQuery.RowAxis),
                                                    CopyAxisWithoutAllMember(newQuery.ColumnAxis),
                                                    CopyAxisWithoutAllMember(newQuery.SlicerAxis),
                                                    newQuery.MdxSettings,
                                                    newQuery.
                                                        CalculatedMembersCache,
                                                    newQuery.Cube);
                    commandText = GenerateCommand(noAllQuery);
                }
                if (CancelPending)
                {
                    return null;
                }

                var gridData = ApplicationServices.CubeExecuteCommand(commandText);

                AxisData columnAxis = null;
                if (query.ColumnAxis != null)
                {
                    columnAxis = gridData.Axes[0];
                }

                AxisData rowAxis = null;
                if (query.RowAxis != null)
                {
                    rowAxis = gridData.Axes[columnAxis == null ? 0 : 1];
                }

                result = GenerateResult(columnAxis, rowAxis, gridData.Cells, query.Cube);
            }
            return result;
        }

        private static MdxAxis CopyAxis(IMdxAxis axis, ObservableCollection<OlapFilter> foreignFilters, bool includeFilters)
        {
            if (axis != null)
            {
            //    if (axis is MdxSlicerAxis)
            //    {
            //        var mdxSlicerAxis = (MdxSlicerAxis)axis;
            //        var unfilteredSlicerAxis = new MdxSlicerAxis(mdxSlicerAxis.MdxSettings);
            //        foreach (var set in axis.GetMdxSets())
            //        {
            //            var clonedSet = set.Clone();
            //            unfilteredSlicerAxis.AddSet(clonedSet);
            //        }
            //        if (includeFilters)
            //            unfilteredSlicerAxis.ColumnsFilter = mdxSlicerAxis.ColumnsFilter;
              
            //        if (includeFilters)
            //            unfilteredSlicerAxis.FiltersFilter = mdxSlicerAxis.FiltersFilter;
            //        unfilteredSlicerAxis.MeasureFilter = mdxSlicerAxis.MeasureFilter;
            //        if (includeFilters)
            //            unfilteredSlicerAxis.RowsFilter = mdxSlicerAxis.RowsFilter;
            //        return unfilteredSlicerAxis;
            //    }
            //    else
            //    {
            //        var unfilteredRowAxis = new MdxAxis(axis.AxisIndex);
            //        foreach (var set in axis.GetMdxSets())
            //        {
            //            var clonedSet = set.Clone();
            //            unfilteredRowAxis.AddSet(clonedSet);
            //        }
            //        return unfilteredRowAxis;
            //    }
            }
            return null;
        }

        private const string AllMember = ".[All]";

        private static MdxAxis CopyAxisWithoutAllMember(IMdxAxis axis)
        {
            if (axis != null)
            {
                //if (axis is MdxSlicerAxis)
                //{
                //    var mdxSlicerAxis = (MdxSlicerAxis)axis;
                //    var s = mdxSlicerAxis.MdxSettings;
                //    var unfilteredSlicerAxis = new MdxSlicerAxis(s);
                //    var mdxSets = axis.GetMdxSets();
                //    foreach (var set in mdxSets)
                //    {
                //        if (set != mdxSets.First() && !set.GetMdxElements().All(m => m.MdxExpression.EndsWith(AllMember)))
                //        {
                //            var clonedSet = set.Clone();
                //            unfilteredSlicerAxis.AddSet(clonedSet);
                //        }
                //    }
                //    unfilteredSlicerAxis.ColumnsFilter = mdxSlicerAxis.ColumnsFilter;
                //    unfilteredSlicerAxis.FiltersFilter = mdxSlicerAxis.FiltersFilter;
                //    unfilteredSlicerAxis.MeasureFilter = mdxSlicerAxis.MeasureFilter;
                //    unfilteredSlicerAxis.RowsFilter = mdxSlicerAxis.RowsFilter;
                //    return unfilteredSlicerAxis;
                //}
                //else
                //{
                //    var unfilteredRowAxis = new MdxAxis(axis.AxisIndex);
                //    var mdxSets = axis.GetMdxSets();
                //    foreach (var set in mdxSets)
                //    {
                //        if (set != mdxSets.First() && !set.GetMdxElements().All(m => m.MdxExpression.EndsWith(AllMember)))
                //        {
                //            var clonedSet = set.Clone();
                //            unfilteredRowAxis.AddSet(clonedSet);
                //        }
                //    }
                //    return unfilteredRowAxis;
                //}
            }
            return null;
        }

        
        private static void FillCellData(ICell[,] cells, IList<CellData> cellsData)
        {
            if (cellsData == null)
            {
                return;
            }

            var columnsCount = cells.GetLength(1);

            for (var i = 0; i < cellsData.Count; i++)
            {
                var cellData = cellsData[i];
                var columnNumber = i % columnsCount;
                var rowNumber = i / columnsCount;

                cells[rowNumber, columnNumber] = new Cell(cellData.Value, cellData.FormattedValue);
            }
        }

        private static Result GenerateResult(AxisData columnAxis, AxisData rowAxis, IList<CellData> cellsData, ICube cube)
        {
            ResultAxis resultColumnAxis = GenerateAxis(columnAxis, cube);

            ResultAxis resultRowAxis = GenerateAxis(rowAxis, cube);

            ICell[,] cells = new ICell[resultRowAxis.Tuples.Count, resultColumnAxis.Tuples.Count];
            FillCellData(cells, cellsData);

            if (resultColumnAxis == ResultAxis.Empty &&
                resultRowAxis == ResultAxis.Empty &&
                cells[0, 0] == null)
            {
                return Result.Empty;
            }

            return new Result(resultRowAxis, resultColumnAxis, cells);
        }

        private static ResultAxis GenerateAxis(AxisData axis, ICube cube)
        {
            List<ITuple> tuples = new List<ITuple>();
            int[] maxDepths;

            if (axis == null || axis.Positions.Count == 0)
            {
                return ResultAxis.Empty;
            }

            IEnumerable<ILevel> levels = cube.Dimensions.SelectMany(d => d.Hierarchies).SelectMany(h => h.Levels);
            PositionData prevTuple = null;
            foreach (PositionData positionData in axis.Positions)
            {
                List<IMember> members = new List<IMember>();
                int memberIndex = 0;

                foreach (MemberData memberData in positionData.Members)
                {
                    bool isTotal = false;
                    if (memberData.DrilledDown)
                    {
                        // member.DrillDownMember = true;
                        isTotal = true;
                    }
                    else
                    {
                        // the member from the previous tuple is child of this member
                        // that does mean it is the Total column and it follows its children members                          
                        if (!memberData.ParentSameAsPrevious &&
                            prevTuple != null &&
                            prevTuple.Members[memberIndex].ParentUniqueName == memberData.UniqueName)
                        {
                            isTotal = true;
                        }
                    }

                    string name = memberData.Name;
                    string uniqueName = memberData.UniqueName;
                    string caption = memberData.Caption;
                    string parentUniqueName = memberData.ParentUniqueName;
                    string levelName = memberData.LevelName;
                    int levelDepth = memberData.LevelDepth;
                    int childCount = memberData.ChildCount;
                    bool drilledDown = memberData.DrilledDown;
                    bool parentSameAsPrevious = memberData.ParentSameAsPrevious;

                    ILevel parentLevel = levels.FirstOrDefault(l => l.UniqueName == levelName);
                    IMember originalMember = null;
                    Member member;
                    if (parentLevel == null || !parentLevel.TryGetMember(uniqueName, out originalMember))
                    {
                        member = new MemberViewModel(
                            name, uniqueName, caption, parentUniqueName, levelName, levelDepth, parentLevel, childCount,
                            drilledDown, parentSameAsPrevious, isTotal);
                    }
                    else
                    {
                        member = new MemberViewModel(
                            name, uniqueName, caption, parentUniqueName, levelName, levelDepth, parentLevel, childCount,
                            drilledDown, parentSameAsPrevious, isTotal, originalMember.UniqueId);
                    }

                    members.Add(member);

                    memberIndex++;
                }

                prevTuple = positionData;

                Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Tuple axisTuple =
                    new Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Tuple(new OlapElementCollection<IMember>(members));

                tuples.Add(axisTuple);
            }

            return new ResultAxis(new ReadOnlyCollection<ITuple>(tuples));
        }
    }
}
