﻿using System;
using System.ComponentModel;
using System.Windows.Media;
using Infragistics.Olap.Data;
using Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008;

namespace OlapAxisWithXMLA
{
    public class MemberViewModel : Member, INotifyPropertyChanged
    {
        public MemberViewModel(string name, string uniqueName, string caption, string parentUniqueName, string levelName, int levelDepth, ILevel parentLevel, int childCount, bool drilledDown, bool parentSameAsPrevious, bool isTotal)
            : base(name, uniqueName, caption, parentUniqueName, levelName, levelDepth, parentLevel, childCount, drilledDown, parentSameAsPrevious, isTotal)
        {
        }

        public MemberViewModel(string name, string uniqueName, string caption, string parentUniqueName, string levelName, int levelDepth, ILevel parentLevel, int childCount, bool drilledDown, bool parentSameAsPrevious, bool isTotal, int uniqueId)
            : base(name, uniqueName, caption, parentUniqueName, levelName, levelDepth, parentLevel, childCount, drilledDown, parentSameAsPrevious, isTotal, uniqueId)
        {
        }

        private bool _isVisible = true;
        public bool IsVisible
        {
            get
            {
                return _isVisible;
            }
            set
            {
                _isVisible = value;
                OnPropertyChanged("IsVisible");
            }
        }

        private string _editableCaption = "";
        public string EditableCaption
        {
            get
            {
                if (string.IsNullOrEmpty(_editableCaption))
                {
                    _editableCaption = Caption;
                    if (string.IsNullOrEmpty(_editableCaption))
                        _editableCaption = "No Value";
                }
                return _editableCaption;
            }
            set
            {
                if (string.IsNullOrEmpty(value))
                    value = "No Value";
                _editableCaption = value;
                OnPropertyChanged("EditableCaption");
            }
        }


        private Color _fill = Colors.Red;
        /// <summary>
        /// color for the DataChart Column series
        /// </summary>
        public Color Fill
        {
            get { return _fill; }
            set
            {
                _fill = value;
                OnPropertyChanged("Fill");
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
        protected void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChangedEventArgs arg = new PropertyChangedEventArgs(propertyName);
                PropertyChanged(this, arg);
            }
        }


        public new bool IsLoaded
        {
            get
            {
                return base.IsLoaded;
            }

            protected internal set
            {
                base.IsLoaded = value;
            }
        }
    }

    public class Level : Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Level
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="Level"/> class.
        /// </summary>
        /// <param name="name">The name of the level.</param>
        /// <param name="caption">The caption of the level.</param>
        /// <param name="uniqueName">The unique name of the level.</param>
        /// <param name="description">The description.</param>
        /// <param name="depth">The distance of the level from the root of the level. Root level is zero (0)</param>
        /// <param name="parentHierarchy">The parent hierarchy.</param>
        /// <param name="membersCount">The count of members of this level.</param>
        public Level(string name, string caption, string uniqueName, string description, int depth, IHierarchy parentHierarchy, int membersCount) :
            base(name, caption, uniqueName, description, depth, parentHierarchy)
        {
            this.MembersCount = membersCount;
        }
    }
}