﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace OlapAxisWithXMLA
{
    public class OlapFilter : INotifyPropertyChanged
    {
        private string _cubeName;
        /// <summary>
        /// 
        /// </summary>
        public string CubeName
        {
            get { return _cubeName; }
            set
            {
                _cubeName = value;
                OnPropertyChanged("CubeName");
            }
        }

        private string _cubeFilter;
        /// <summary>
        /// 
        /// </summary>
        public string CubeFilter
        {
            get { return _cubeFilter; }
            set
            {
                _cubeFilter = value;
                OnPropertyChanged("CubeFilter");
            }
        }

        private string _fieldName;
        /// <summary>
        /// 
        /// </summary>
        public string FieldName
        {
            get { return _fieldName; }
            set
            {
                _fieldName = value;
                OnPropertyChanged("FieldName");
            }
        }

        private object _fieldValue;
        /// <summary>
        /// 
        /// </summary>
        public object FieldValue
        {
            get { return _fieldValue; }
            set
            {
                _fieldValue = value;
                OnPropertyChanged("FieldValue");
            }
        }

        private string _fieldCaption;
        /// <summary>
        /// 
        /// </summary>
        public string FieldCaption
        {
            get { return _fieldCaption; }
            set
            {
                _fieldCaption = value;
                OnPropertyChanged("FieldCaption");
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public int RowIndex { get; set; }
        /// <summary>
        /// 
        /// </summary>
        public int ColIndex { get; set; }

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string prop)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(prop));
        }
    }

    public class XmlaElement
    {
        public string CatalogName { get; set; }
        public string CubeName { get; set; }
        public string Description { get; set; }
    }

    public class XmlaStyledElement : XmlaElement
    {
        public ColorInfo RGB { get; set; }
    }

    public class ColorInfo
    {
        public byte A { get; set; }
        public byte B { get; set; }
        public byte G { get; set; }
        public byte R { get; set; }
    }

    public class AdomdHierarchy : XmlaElement
    {
        public string AllMember { get; set; }
        public string Caption { get; set; }
        public string DefaultMember { get; set; }
        public string DimensionUniqueName { get; set; }
        List<AdomdLevel> levels = new List<AdomdLevel>();
        public List<AdomdLevel> Levels { get { return levels; } }
        public string Name { get; set; }
        public string UniqueName { get; set; }
    }

    public class AdomdLevel : XmlaElement
    {
        public string DimensionUniqueName { get; set; }

        public string HierarchyUniqueName { get; set; }

        public string Name { get; set; }

        public string UniqueName { get; set; }

        public string Caption { get; set; }

        public uint Number { get; set; }

        public long MembersCount { get; set; }
    }

    public class AdomdCube : XmlaElement
    {
        public string Caption { get; set; }
        public string CubeType { get; set; }
        public DateTime LastDataUpdate { get; set; }
        public DateTime LastSchemaUpdate { get; set; }
    }

    public class AdomdMeasure : XmlaElement
    {
        public string Name { get; set; }

        public string UniqueName { get; set; }

        public string Caption { get; set; }

        public string Aggregator { get; set; }

        public string MeasureGroupName { get; set; }

        public string DefaultFormatString { get; set; }
    }
    
    public class AdomdDimension : XmlaStyledElement
    {
        public string Caption { get; set; }
        public string DimensionType { get; set; }
        List<AdomdHierarchy> hierarchies = new List<AdomdHierarchy>();
        public List<AdomdHierarchy> Hierarchies { get { return hierarchies; } }
        public string Name { get; set; }
        public string UniqueName { get; set; }
    }

    public class AdomdMember : XmlaStyledElement
    {
        public string Caption { get; set; }
        public int ChildrenCardinality { get; set; }
        public uint LevelNumber { get; set; }
        public string LevelUniqueName { get; set; }
        public string Name { get; set; }
        public string ParentUniqueName { get; set; }
        public string UniqueName { get; set; }
    }

    public class GridData
    {
        public List<AxisData> Axes { get; set; }
        public List<CellData> Cells { get; set; }
    }

    public class CellData
    {
        public string FormattedValue { get; set; }
        public object Value { get; set; }
    }

    public class AxisData
    {
        public List<PositionData> Positions { get; set; }
    }

    public class PositionData
    {
        public List<MemberData> Members { get; set; }
        public int Ordinal { get; set; }
    }

    public class MemberData
    {
        public string Caption { get; set; }
        public int ChildCount { get; set; }
        public bool DrilledDown { get; set; }
        public int LevelDepth { get; set; }
        public string LevelName { get; set; }
        public string Name { get; set; }
        public bool ParentSameAsPrevious { get; set; }
        public string ParentUniqueName { get; set; }
        public string UniqueName { get; set; }
    }

}