﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Windows.Input;
using Infragistics.Olap;
using Infragistics.Olap.Data;
using Infragistics.Olap.Data.Base;
using Infragistics.Olap.Xmla;
using System;

namespace OlapAxisWithXMLA
{
    public class XmlaDataSourceExt : XmlaDataSource
    {
        /// <summary>
        /// 
        /// </summary>
        public XmlaDataSourceExt()
        {
            ((IOlapViewModel)this).Columns.CollectionChanged += ColumnsCollectionChanged;
            ((IOlapViewModel)this).Rows.CollectionChanged += ColumnsCollectionChanged;
            PropertyChanged += new PropertyChangedEventHandler(XmlaDataSourceExt_PropertyChanged);


            Cube = XmlaDataSource.GenerateInitialCube("Adventure Works");
            Rows = XmlaDataSource.GenerateInitialItems("[Date].[Calendar]");
            Columns = XmlaDataSource.GenerateInitialItems("[Geography].[City]");
            Measures = XmlaDataSource.GenerateInitialItems("Reseller Sales Amount");
        }

        void XmlaDataSourceExt_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {

        }

        private ICommand _UpdateSourceCommand;
        /// <summary>
        /// 
        /// </summary>
        public ICommand UpdateSourceCommand
        {
            get
            {
                if (_UpdateSourceCommand == null)
                    _UpdateSourceCommand = new RelayCommand(DoUpdateSourceCommand);

                return _UpdateSourceCommand;
            }
        }

        //WidgetType selectedWidgetType = WidgetType.PivotGrid;
        //public WidgetType SelectedWidgetType
        //{
        //    get;
        //    set;
        //}

        private void DoUpdateSourceCommand(object defaultValue)
        {
            ExpandAll();
        }

        /// <summary>
        /// The query result without any filters
        /// </summary>
        public IResult UnfilteredResult { get; set; }

        /// <summary>
        /// The query result without any filters
        /// </summary>
        public bool PopulateIsTotalProperty { get; set; }

        /// <summary>
        /// this hold the pivotgrid layout object
        /// </summary>
        public Infragistics.Controls.Grids.PivotGridLayout AIRPivotGridLayout { get; set; }

        /// <summary>
        /// Called when all results change
        /// </summary>
        protected virtual void OnAllResultsChanged()
        {
        }

        /// <summary>
        /// Data Provider
        /// </summary>
        public AdomdDataProvider AdomdDataProvider
        {
            get { return DataProvider as AdomdDataProvider; }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        protected override XmlaCommand GetXmlaCommand()
        {
            var command = base.GetXmlaCommand();
            // apply the geographic filter
            if (command is AdomdCommand)
            {
                ((AdomdCommand)command).ForeignFilters = _foreignFilters;
            }

            return command;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="args"></param>
        protected override void OnResultChanged(ResultChangedEventArgs args)
        {
            if (PopulateIsTotalProperty)
            {
                // reset the isTotal properties since they get erased
                UpdateIsTotalProperties(Result.RowAxis);
                UpdateIsTotalProperties(Result.ColumnAxis);
            }

            base.OnResultChanged(args);
        }

        private ObservableCollection<IFilterViewModel> _selectedFilterViewModels;


        /// <summary>
        /// A list of all selected filter view models in the data source
        /// </summary>
        public ObservableCollection<IFilterViewModel> SelectedFilterViewModels
        {
            get
            {
                if (_selectedFilterViewModels == null)
                {
                    _selectedFilterViewModels = new ObservableCollection<IFilterViewModel>();
                    UpdateSelectedFilterViewModels();
                }
                return _selectedFilterViewModels;
            }
        }

        private void UpdateSelectedFilterViewModels()
        {
            var remove =
                SelectedFilterViewModels.Where(vm => !Columns.Any(c => c == vm) && !Rows.Any(c => c == vm)).ToList();
            foreach (var vm in remove)
                SelectedFilterViewModels.Remove(vm);
            foreach (var filterViewModel in Columns.Where(c => c is IFilterViewModel && !SelectedFilterViewModels.Any(sfvm => sfvm == c)))
            {
                if (SelectedFilterViewModels.Count > Columns.IndexOf(filterViewModel))
                    SelectedFilterViewModels.Insert(Columns.IndexOf(filterViewModel), (IFilterViewModel)filterViewModel);
                else
                    SelectedFilterViewModels.Insert(SelectedFilterViewModels.Count, (IFilterViewModel)filterViewModel);
            }
            foreach (var filterViewModel in Rows.Where(c => c is IFilterViewModel && !SelectedFilterViewModels.Any(sfvm => sfvm == c)))
            {
                if (SelectedFilterViewModels.Count > Columns.Count + Rows.IndexOf(filterViewModel))
                    SelectedFilterViewModels.Insert(Columns.Count + Rows.IndexOf(filterViewModel), (IFilterViewModel)filterViewModel);
                else
                    SelectedFilterViewModels.Insert(SelectedFilterViewModels.Count, (IFilterViewModel)filterViewModel);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="hierarchy"></param>
        /// <returns></returns>
        public override IFilterViewModel CreateFilterViewModel(IHierarchy hierarchy)
        {
            var originalViewModel = base.CreateFilterViewModel(hierarchy);
            var retVal = originalViewModel;
            // change the type to FilterViewModelBaseExt so we have more control to add properties
            if (retVal is FilterViewModelBase)
            {
                retVal = new FilterViewModelBaseExt(retVal.ViewModel, retVal.DataProvider, retVal.ViewModel.Cube, hierarchy)
                {
                    IsAllSelected = originalViewModel.IsAllSelected,
                    IsDirty = originalViewModel.IsDirty,
                    ParentInFront = originalViewModel.ParentInFront,
                };
                retVal.LoadFilterMembersAsync();
            }
            return retVal;
        }


        private ICommand _toggleMemberVisibilityCommand;
        /// <summary>
        /// 
        /// </summary>
        public ICommand ToggleMemberVisibilityCommand
        {
            get
            {
                if (_toggleMemberVisibilityCommand == null)
                    _toggleMemberVisibilityCommand = new RelayCommand(OnToggleMemberVisibilityCommand);

                return _toggleMemberVisibilityCommand;
            }
        }
        /// <summary>
        /// 
        /// </summary>
        public void OnToggleMemberVisibilityCommand(object defaultValue)
        {
            var member = defaultValue as IFilterMember;
            if (member != null && member.IsSelected.HasValue && member.Member is MemberViewModel)
            {
                member.IsSelected = !member.IsSelected.Value;
                ((MemberViewModel)member.Member).IsVisible = member.IsSelected.Value;
                RefreshGrid();
            }
        }

        private void UpdateIsTotalProperties(IResultAxis resultAxis)
        {
            for (var i = 0; i < resultAxis.Tuples.Count; i++)
            {
                ITuple prevTuple = null;
                if (i > 0)
                    prevTuple = resultAxis.Tuples[i - 1];
                var tuple = resultAxis.Tuples[i];

                tuple.IsTotal = false;
                if (tuple.Members != null && tuple.Members.Count > 0 && prevTuple != null)
                {
                    // if this is the parent of the previous member, then it is a total
                    if ((tuple.Members.Any(m => !m.ParentSameAsPrevious
                                                && prevTuple.Members[tuple.Members.IndexOf(m)].ParentMember != null
                                                &&
                                                prevTuple.Members[tuple.Members.IndexOf(m)].ParentMember.UniqueName ==
                                                m.UniqueName)
                        )
                        // if this begins with "all" then it is in the grand total section
                        || tuple.Members.First().Caption.ToLower() == "all"
                        )
                    {
                        tuple.IsTotal = true;
                    }
                    else if (tuple.Members.Any(m => m.Caption.ToLower() == "all"))
                    {
                        // while this is a subtotal section
                        // make sure this isn't going to wipe out the parent member
                        var tupleMembersUpToFirstAllMember =
                            tuple.Members.TakeWhile(m => m.Caption == null || m.Caption.ToLower() != "all");
                        var previousTuples = Result.RowAxis.Tuples.Where(t =>
                                                                         t != tuple &&
                                                                         t.Members.First().Caption ==
                                                                         tuple.Members.First().Caption
                                                                         && !t.IsTotal
                                                                         &&
                                                                         (t.Members.Count <=
                                                                          tupleMembersUpToFirstAllMember.Count()
                                                                          ||
                                                                          (t.Members[
                                                                              tupleMembersUpToFirstAllMember.Count()
                                                                               ].Caption == null ||
                                                                           t.Members[
                                                                               tupleMembersUpToFirstAllMember.Count()
                                                                               ].Caption.ToLower() != "all")));
                        if (
                            previousTuples.Any(
                                pt =>
                                tupleMembersUpToFirstAllMember.All(
                                    tm => pt.Members.Any(ptmem => ptmem.Caption == tm.Caption))))
                            tuple.IsTotal = true;
                    }
                }
            }
        }


        void ExecuteMultipleCommandsCompleted(object sender, ExecuteMultipleCommandsCompletedEventArgs e)
        {
            // we have multiple results
            //UnfilteredResult = e.ExecuteCommandCompletedEventArgs[1].Result;
            OnAllResultsChanged();
        }


        /// <summary>
        /// 
        /// </summary>
        public bool ExpandingNodes { get; set; }

        void ColumnsCollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            ExpandAll();
            // notify that the selected filter view models has changed
            UpdateSelectedFilterViewModels();
        }

       
        bool isSwapAxis = false;
        /// <summary>
        /// swap axis only once for a widget.
        /// </summary>
        public void SwapAxis()
        {
            if (!isSwapAxis)
            {
                PivotGrid();
                if (Cube != null)
                    isSwapAxis = true;
            }
        }

        private NodeExpansionBehavior _expandAllNodesOnUpdate = NodeExpansionBehavior.None;
        /// <summary>
        /// 
        /// </summary>
        public NodeExpansionBehavior ExpandAllNodesOnUpdate
        {
            get { return _expandAllNodesOnUpdate; }
            set { _expandAllNodesOnUpdate = value; }
        }

      

        internal void ExpandAll()
        {
            if (ExpandAllNodesOnUpdate != NodeExpansionBehavior.None && !ExpandingNodes)
            {
                ExpandingNodes = true;

                var manuallyDeferredUpdate = false;

                if (!DeferredLayoutUpdate)
                {
                    manuallyDeferredUpdate = true;
                }


                if (Rows.Count > 0)
                {
                    for (var i = 0;
                         i <
                         (ExpandAllNodesOnUpdate == NodeExpansionBehavior.First ? 1 : Rows.Count);
                         i++)
                    {
                        var filterViewModel = Rows[i] as IFilterViewModel;
                        ExpandAttribute(filterViewModel);
                    }
                }

                if (Columns.Count > 0)
                {
                    for (var i = 0;
                         i <
                         (ExpandAllNodesOnUpdate == NodeExpansionBehavior.First ? 1 : Columns.Count);
                         i++)
                    {
                        var filterViewModel = Columns[i] as IFilterViewModel;
                        ExpandAttribute(filterViewModel);
                    }
                }

                ResultChanged += DataSourceResultChanged;

                if (manuallyDeferredUpdate && DeferredLayoutUpdate)
                    DeferredLayoutUpdate = false;
                ExpandingNodes = false;
            }
        }

        static void DataSourceResultChanged(object sender, AsyncCompletedEventArgs e)
        {
            var dataSourceBase = (DataSourceBase)sender;
            dataSourceBase.ResultChanged -= DataSourceResultChanged;

            dataSourceBase.HierarchyExpansionMode = HierarchyExpansionMode.DrillDown;
            dataSourceBase.UpdateVisualMembers(ExecutionContext.Rows);
            dataSourceBase.UpdateVisualMembers(ExecutionContext.Columns);
            dataSourceBase.HierarchyExpansionMode = HierarchyExpansionMode.SingleLevelOnly;

            dataSourceBase.RefreshGrid();
        }

        private void ExpandAttribute(IFilterViewModel filterViewModel, bool deferUpdate = true)
        {
            if (filterViewModel != null)
            {
                if (filterViewModel.FilterMembers != null)
                {
                    foreach (var member in filterViewModel.FilterMembers)
                    {
                        DrillIn(member, deferUpdate);
                    }
                }
            }
        }

        private void DrillIn(IFilterMember filterMember, bool deferUpdate = true)
        {

            //Condition to drill in further
            if (filterMember.Member.LevelDepth < 15)
            {
                if (!filterMember.IsExpanded)
                {
                    if (deferUpdate && !DeferredLayoutUpdate)
                        DeferredLayoutUpdate = true;
                    filterMember.IsExpanded = true;
                }

                //if (filterMember.FilterMembers != null)
                //{
                //    foreach (var member in filterMember.FilterMembers)
                //    {
                //        //DrillIn(filterViewModel, member);
                //    }
                //}
            }
            //else
            //{
            //    filterMember.IsExpanded = false;
            //}
        }

        private ICommand _requestChangeAggregate;

        /// <summary>
        /// 
        /// </summary>
        public ICommand RequestChangeAggregate
        {
            get
            {
                if (_requestChangeAggregate == null)
                    _requestChangeAggregate = new RelayCommand(OnChangeAggregate);

                return _requestChangeAggregate;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="parameter"></param>
        public void OnChangeAggregate(object parameter)
        {
            var parameterList = parameter as List<string>;
            if (parameterList == null || parameterList.Count <= 0)
                return;

            var aggregate = parameterList[0];
            var attribute = parameterList[1];

            var measure = Measures.SingleOrDefault(m => m.Caption.ToLower() == attribute.ToLower());
            if (measure == null)
                return;
            int index = Measures.IndexOf(measure);
            Measures.Remove(measure);
            if (attribute.ToLower().EndsWith("min") || attribute.ToLower().EndsWith("max") || attribute.ToLower().EndsWith("sum"))
            {
                attribute = attribute.Substring(0, attribute.Length - 3);
            }

            var hierarchy = Cube.Measures.FirstOrDefault(m => m.Caption.ToLower().StartsWith(attribute.ToLower())
                && m.Caption.ToLower().EndsWith(aggregate.ToLower()));
            if (hierarchy != null)
            {
                var measureViewModel = CreateMeasureViewModel(hierarchy);

                Measures.Insert(index, measureViewModel);
            }

        }

       
        private readonly List<IFilterMember> _userSelectedFilterMembers = new List<IFilterMember>();

        internal IFilterMember GetMatchingFilterMember(IEnumerable<IFilterMember> filterMembers, IFilterMember matchingFilterMember)
        {
            IFilterMember retVal = null;
            foreach (var filterMember in filterMembers)
            {
                if (filterMember.UniqueName == matchingFilterMember.UniqueName)
                {
                    retVal = filterMember;
                    break;
                }

                if (filterMember.FilterMembers != null && filterMember.FilterMembers.Count > 0)
                {
                    retVal = GetMatchingFilterMember(filterMember.FilterMembers, matchingFilterMember);
                    if (retVal != null)
                        break;
                }
            }
            return retVal;
        }

        private readonly ObservableCollection<OlapFilter> _foreignFilters = new ObservableCollection<OlapFilter>();

        /// <summary>
        /// Indicates if an enumerable of custom olap filters are being applied to this xaml data source
        /// </summary>
        public bool ApplyingCustomOlapFilter { get; set; }

        // keep up with what gets turned on and off
        // as a result of applying an olap filter so
        // only those will be toggled when removing 
        // the filter

        internal bool AddFilters(IEnumerable<OlapFilter> filters)
        {
            ApplyingCustomOlapFilter = true;
            var retVal = false;
            foreach (var filter in filters)
            {
                var filter1 = filter;
                var cubeFilter =
                    (Rows.SingleOrDefault(r => r is IFilterViewModel && ((IFilterViewModel)r).Hierarchy.Name == filter1.FieldName) as IFilterViewModel ??
                     Columns.SingleOrDefault(r => r is IFilterViewModel && ((IFilterViewModel)r).Hierarchy.Name == filter1.FieldName) as IFilterViewModel) ??
                    Filters.SingleOrDefault(r => r is IFilterViewModel && ((IFilterViewModel)r).Hierarchy.Name == filter1.FieldName) as IFilterViewModel;
                if (cubeFilter == null && Cube != null)
                {
                    // add it to filters if it is still missing
                    var dimHierarchy = Cube.Dimensions.Where(d => d.Hierarchies.Any(h => h.Name == filter1.FieldName)).First().Hierarchies[filter1.FieldName];

                    cubeFilter = CreateFilterViewModel(dimHierarchy);
                    Filters.Add(cubeFilter);
                    cubeFilter.FilterMembers.First().IsExpanded = true;
                }

                var allMember = cubeFilter.FilterMembers.First();

                if (allMember.IsLoaded)
                {
                    var member =
                        allMember.FilterMembers.SingleOrDefault(f => f.Name ==
                                                                        filter1.FieldValue.ToString()
                                                                ||
                                                                    (
                            // removed because some values are legitimately empty
                            //!string.IsNullOrEmpty(filter1.FieldCaption) &&
                                                                        f.Name == filter1.FieldCaption.ToString()));


                    if (member != null)
                    {
                        // if they are all selected, then turn them all off so the filter will apply
                        //if (allMember.FilterMembers.All(m1 => m1.IsSelected.HasValue && m1.IsSelected.Value))
                        //{
                        foreach (var member1 in allMember.FilterMembers.Where(fm =>
                            fm != member && fm.IsSelected.HasValue && fm.IsSelected.Value
                                //    // make sure we aren't turning something off that is a part of a 
                                //    // pre-existing filter
                            && !_foreignFilters.Any(ff => ff.FieldName == filter1.FieldName
                                                && fm.Name == ff.FieldValue.ToString()
                                                                ||
                                                                    (
                                // removed because some values are legitimately empty
                                //!string.IsNullOrEmpty(ff.FieldCaption) && 
                                                                    fm.Name == ff.FieldCaption.ToString()))
                                // make sure it is not a manual user filter
                            && !_userSelectedFilterMembers.Any(f => f.UniqueName == fm.UniqueName)
                            ))
                        {
                            retVal = true;
                            member1.IsSelected = false;
                        }
                        //}
                        // apply the filter
                        // only if it is not a prebiously defined user manual filter
                        if ((!member.IsSelected.HasValue || !member.IsSelected.Value) && !_userSelectedFilterMembers.Any(f => f.UniqueName == member.UniqueName))
                        {
                            retVal = true;
                            member.IsSelected = true;
                        }
                    }
                }
                _foreignFilters.Add(filter);
            }
            ApplyingCustomOlapFilter = false;
            return retVal;
        }

        internal bool RemoveFilters(IEnumerable<OlapFilter> filters)
        {
            ApplyingCustomOlapFilter = true;
            var retVal = false;
            foreach (var filter in filters)
            {
                var filter1 = filter;
                var cubeFilter =
                    (Rows.SingleOrDefault(r => r is IFilterViewModel && ((IFilterViewModel)r).Hierarchy.Name == filter1.FieldName) as IFilterViewModel ??
                     Columns.SingleOrDefault(r => r is IFilterViewModel && ((IFilterViewModel)r).Hierarchy.Name == filter1.FieldName) as IFilterViewModel) ??
                    Filters.SingleOrDefault(r => r is IFilterViewModel && ((IFilterViewModel)r).Hierarchy.Name == filter1.FieldName) as IFilterViewModel;
                if (cubeFilter != null)
                {
                    var allMember = cubeFilter.FilterMembers.First();

                    if (allMember.IsLoaded)
                    {
                        var allSubMembers = allMember.FilterMembers;

                        var member =
                            allSubMembers.SingleOrDefault(f => f.Name == filter1.FieldValue.ToString()
                                                                ||
                                                                    (
                                // removed because some values are legitimately empty
                                //!string.IsNullOrEmpty(filter1.FieldCaption) && 
                                                                        f.Name == filter1.FieldCaption.ToString()));

                        // only set it back to false if it is not part of a filter applied manually by the user
                        if (member != null && member.IsSelected == true && !_userSelectedFilterMembers.Any(f => f.UniqueName == member.UniqueName))
                        {
                            // remove the filter
                            member.IsSelected = false;
                            retVal = true;
                        }

                        // if they are all unselected, then this was just a filter that has 
                        // been now removed and so everything should be turned back on
                        if (allSubMembers.All(m1 => m1.IsSelected.HasValue && !m1.IsSelected.Value))
                        {
                            foreach (var member1 in allSubMembers.Where(f => !_userSelectedFilterMembers.Any(f2 => f.UniqueName == f2.UniqueName)))
                            {
                                member1.IsSelected = true;
                                retVal = true;
                            }
                            // if they are all turned on and its in the filter box, remove it
                            if (allSubMembers.All(m1 => m1.IsSelected.HasValue && m1.IsSelected.Value) && Filters.Contains(cubeFilter))
                            {
                                Filters.Remove(cubeFilter);
                            }
                        }


                    }
                }
                _foreignFilters.Remove(filter);
            }
            ApplyingCustomOlapFilter = false;
            return retVal;
        }

        /// <summary>
        /// Sets the selected dimension attributes to a single one
        /// </summary>
        /// <param name="selectedDimension"></param>
        /// <param name="dimensionAttributeType"></param>
        /// <returns></returns>
        public IFilterViewModel SetSingleSelectedDimensionAttribute(HierarchicalItem selectedDimension, DimensionAttributeType dimensionAttributeType)
        {
            if (selectedDimension != null)
            {
                var name = "";

                if (selectedDimension.DataObject is Hierarchy)
                    name = ((Hierarchy)selectedDimension.DataObject).Name;
                else if (selectedDimension.DataObject is Dimension)
                    name = ((Dimension)selectedDimension.DataObject).Name;

                var matchingDimensionAttributes =
                    Cube.Dimensions.Where(d => d.Hierarchies.Any(h => h.Name == name));

                if (matchingDimensionAttributes.Count() > 0)
                {
                    // need to do this in deferred update mode
                    var update = !DeferredLayoutUpdate;
                    if (update)
                    {
                        DeferredLayoutUpdate = true;
                    }

                    ExpandAllNodesOnUpdate = NodeExpansionBehavior.None;
                    var hierarchy = matchingDimensionAttributes.First().Hierarchies[name];
                    var filterViewModel = CreateFilterViewModel(hierarchy);
                    filterViewModel.FilterMembers.First().IsExpanded = true;

                    if (Filters.Any(f => f.Caption == filterViewModel.Caption))
                        Filters.RemoveRange(Filters.Where(f => f.Caption == filterViewModel.Caption).ToList());

                    if (dimensionAttributeType == DimensionAttributeType.Column)
                    {

                        if (Rows.Any(f => f.Caption == filterViewModel.Caption))
                            Rows.RemoveRange(Filters.Where(f => f.Caption == filterViewModel.Caption).ToList());

                        for (var i = Columns.Count - 1; i >= 0; i--)
                        {
                            Columns.RemoveAt(i);
                        }
                        Columns.Add(filterViewModel);
                    }
                    else
                    {

                        if (Columns.Any(f => f.Caption == filterViewModel.Caption))
                            Columns.RemoveRange(Filters.Where(f => f.Caption == filterViewModel.Caption).ToList());

                        for (var i = Rows.Count - 1; i >= 0; i--)
                        {
                            Rows.RemoveAt(i);
                        }
                        Rows.Add(filterViewModel);
                    }

                    if (update)
                    {
                        DeferredLayoutUpdate = false;
                    }

                    return filterViewModel;
                }
            }

            return null;
        }

        /// <summary>
        /// Sets a single measure based on a hierarchical item
        /// </summary>
        /// <param name="selectedMeasure"></param>
        /// <param name="selectedValueAggregate"></param>
        internal void SetSelectedMeasure(HierarchicalItem selectedMeasure, string selectedValueAggregate)
        {
            if (selectedMeasure != null)
            {
                var name = "";

                if (selectedMeasure.DataObject is Hierarchy)
                    name = ((Hierarchy)selectedMeasure.DataObject).Name;
                else if (selectedMeasure.DataObject is Dimension)
                    name = ((Dimension)selectedMeasure.DataObject).Name;
                else if (selectedMeasure.DataObject is Measure)
                    name = ((Measure)selectedMeasure.DataObject).Name;
                else if (selectedMeasure.DataObject is MeasureGroup)
                    name = ((MeasureGroup)selectedMeasure.DataObject).Name;

                // need to do this in deferred update mode
                ExpandAllNodesOnUpdate = NodeExpansionBehavior.None;
                var hierarchy = Cube.Measures[name
                                                              + (selectedValueAggregate == "Total" ? "Sum" : selectedValueAggregate)] ??
                                Cube.Measures[name];

                SetSelectedMeasure(hierarchy);
            }
        }

        /// <summary>
        /// Sets a single measure based on an object IMeasure
        /// </summary>
        /// <param name="measure"></param>
        internal void SetSelectedMeasure(IMeasure measure)
        {
            if (measure != null)
            {
                var update = !DeferredLayoutUpdate;
                if (update)
                {
                    DeferredLayoutUpdate = true;
                }

                var measureViewModel = CreateMeasureViewModel(measure);

                for (var i = Measures.Count - 1; i >= 0; i--)
                {
                    Measures.RemoveAt(i);
                }
                Measures.Clear();
                Measures.Add(measureViewModel);

                if (update)
                {
                    DeferredLayoutUpdate = false;
                }
            }
        }

        internal string GetPlainTextTitle()
        {
            var title = "";

            var result = Result;

            if (result != null)
            {

                var measures = "";
                // get measures
                foreach (var measure in Measures)
                {
                    if (measures.Length > 0)
                    {
                        if (measure == Measures.Last())
                            measures += " and ";
                        else
                            measures += ", ";
                    }
                    measures += measure.Caption;
                }

                title = measures + " for ";

                var attributes = "";
                foreach (var attribute in Rows)
                {
                    if (attributes.Length > 0)
                    {
                        if (attribute == Rows.Last())
                            attributes += " and ";
                        else
                            attributes += ", ";
                    }
                    attributes += attribute.Caption;
                }

                var columnAttributes = "";
                foreach (var attribute in Columns)
                {
                    if (columnAttributes.Length > 0)
                    {
                        if (attribute == Columns.Last())
                            columnAttributes += " and ";
                        else
                            columnAttributes += ", ";
                    }
                    columnAttributes += attribute.Caption;
                }

                if (columnAttributes.Length > 0 && attributes.Length > 0)
                    attributes += " by ";

                if (columnAttributes.Length > 0)
                    attributes += columnAttributes;

                if (attributes.Length == 0)
                    attributes = "all records";

                title += attributes;

            }
            return title;
        }

    }

    public enum NodeExpansionBehavior
    {
        None,
        All,
        First
    }

    public enum WidgetType
    {
        PivotGrid,
        Chart
    }
    public enum DimensionAttributeType
    {
        Column,
        Row
    }
}
